# Overview

[![Pub](https://img.shields.io/pub/v/ms_list_utils.svg)](https://pub.dartlang.org/packages/ms_list_utils)
[![Pub](./coverage_badge.svg)](https://github.com/Marcus-Software/ms_list_utils/tree/master/test)
[![GitHub stars](https://img.shields.io/github/stars/Marcus-Software/ms_list_utils?style=social)](https://github.com/Marcus-Software/ms_list_utils)
<span class="badge-buymeacoffee">
<a href="https://www.buymeacoffee.com/marcusedu" title="Donate to this project using Buy Me A Coffee"><img src="https://img.shields.io/badge/buy%20me%20a%20coffee-donate-yellow.svg" alt="Buy Me A Coffee donate button" /></a>
</span>

Some help functions for using Lists

## Whats MS_List_Utils do

Add useful functions to map:

* [`addAround`](#addAround) Returns new list with items generated around list.
* [`addBetween`](#addBetween) Returns new list with items generated between list.
* [`containsAll`](#containsAll) Returns new list with items generated between list.
* [`containsAny`](#containsAny) Returns new list with items generated between list.
* [`containsAtLeast`](#containsAtLeast) Returns new list with items generated between list.
* [`containsHits`](#containsHits) Returns new list with items generated between list.
* [`diff`](#addBetween) Elements that you have in listA and do not have in listB.
* [`firstOrNull`](#firstOrNull) Returns first element in array, if it's empty returns null.
* [`firstWhereOrAdd`](#firstWhereOrAdd) Returns the first element that satisfies the test if there isn't one add a new
  one and return it.
* [`flat`](#flat) Returns a flatted a [list] that has other lists inside other lists inside other lists ... recursive
* [`intersection`](#flat) Common elements between listA and listB
* [`isFirst`](#isFirst) Returns a true value if element is the first in the list.
* [`isLast`](#isLast) Returns a true value if element is the last in the list.
* [`joinLast`](#joinLast) It joins all elements of the list with one separator and for the last iteration a different
  separator.
* [`lastOrNull`](#lastOrNull) Returns last element in array, if it's empty returns null.
* [`lastWhereOrAdd`](#lastWhereOrAdd) Returns the last element that satisfies the test if there isn't one add a new
  one and return it.
* [`toMap`](#toMap) Returns a Map with keys that's generated by generator.
* [`toStream`](#toStream) Creates a stream with the items in the list.

## Usage

Just import lib and use [extensions](https://dart.dev/guides/language/extension-methods), call the functions to starts
work:

``` dart
// Don't forget to import
import 'package:ms_list_utils/ms_list_utils.dart';

List list = ['😁','😒','😊'];
final newList = addAround(list, (index,previous,next) => '🍨');
print(newList);//['🍨','😁','🍨','😒','🍨','😊','🍨']
// Or use extensions
final newList = list.addAround((index,previous,next) => '🍨');
print(newList);//['🍨','😁','🍨','😒','🍨','😊','🍨']
```

### <a name="addAround"></a>addAround

The function `addAround` returns new list with items generated around list.

``` dart
  test('tests the function add_around.dart', () {
    final initialValue = [1, 2, 3, 4];
    final result = addAround(initialValue,
        (int index, int? previousValue, int? nextValue) {
      return 5;
    });
    expect(result, [5, 1, 5, 2, 5, 3, 5, 4, 5]);
    expect(initialValue.addAround((index, previous, next) => 5),
        [5, 1, 5, 2, 5, 3, 5, 4, 5]);
  });
```

see more in [test file](./test/add_around_test.dart).

### <a name="addBetween"></a>addBetween

The function `addBetween` returns new list with items generated between list.

``` dart
  test('add values between in list', () {
    final initialValue = <int>[1, 2, 3];
    final result = addBetween<int>(
        initialValue, (int index, previousValue, nextValue) => 5);
    expect(result, [1, 5, 2, 5, 3]);
    expect(
        initialValue.addBetween((index, previous, next) => 5), [1, 5, 2, 5, 3]);
  });
```

see more in [test file](./test/add_between_test.dart).

### <a name="diff"></a>addBetween

The function `diff` returns elements that you have in listA and do not have in listB.

``` dart
  test('differences between two lists', () {
    final listA = [1, 2, 3, 4, 5];
    final listB = [2, 4, 6];
    expect(diff(listA, listB), orderedEquals([1, 3, 5]));
    expect(diff(listB, listA), orderedEquals([6]));
    expect(listB - listA, orderedEquals([6]));
    expect(listB.diff(listA), orderedEquals([6]));
  });
```

see more in [test file](./test/diff_test.dart).

### <a name="firstWhereOrAdd"></a>firstWhereOrAdd

The function `firstWhereOrAdd` returns the first element that satisfies the test if there isn't one add a new one and
return it.

``` dart
  test('tests the function first_where_or_add.dart', () {
    final list = [1, 2, 3];
    expect(firstWhereOrAdd(list, (value) => value == 4, () => 4), 4);
    expect(list, [1, 2, 3, 4]);
    final listE = [1, 2, 3];
    expect(listE.firstWhereOrAdd((value) => value == 4, () => 4), 4);
    expect(listE, [1, 2, 3, 4]);
  });
```

### <a name="flat"></a>flat

The function `flat` returns a flatted a [list] that has other lists inside other lists inside other lists ... recursive
return it.

``` dart
  test('flat dynamic', () {
    final multiList = [
      1,
      2,
      3,
      [
        "4",
        "5",
        [
          6,
          7,
          [true, true, false]
        ]
      ]
    ];
    final result = flat(multiList);
    expect(result, [1, 2, 3, "4", "5", 6, 7, true, true, false]);
    expect(multiList.flat(), [1, 2, 3, "4", "5", 6, 7, true, true, false]);
  });
```

### <a name="intersection"></a>flat

The function `intersection` returns common items between lists

``` dart
  test('intersection between two lists', () {
    final listA = [1, 2, 3, 4, 5];
    final listB = [2, 4, 6];
    expect(intersection(listA, listB), orderedEquals([2, 4]));
    expect(listA.intersection(listB), orderedEquals([2, 4]));
  });
```

see more in [test file](./test/intersection_test.dart).

### <a name="isFirst"></a>isFirst

The function `isFirst` returns a true value if element is the first in the list.

``` dart
  test('tests the function is_first.dart', () {
    final list = [1, 2, 3];
    expect(isFirst(list, 3), isFalse);
    expect(isFirst(list, 1), isTrue);
    expect(isFirstIndex(list, 3), isFalse);
    expect(isFirstIndex(list, 0), isTrue);
    expect(list.isFirst(3), isFalse);
    expect(list.isFirst(1), isTrue);
    expect(list.isFirstIndex(3), isFalse);
    expect(list.isFirstIndex(0), isTrue);
  });
```

see more in [test file](./test/is_first_test.dart).

### <a name="isLast"></a>isLast

The function `isLast` returns a true value if element is the last in the list.

``` dart
  test('tests the function is_last.dart', () {
    final list = [1, 2, 3];
    expect(isLast(list, 1), isFalse);
    expect(isLast(list, 3), isTrue);
    expect(isLastIndex(list, 1), isFalse);
    expect(isLastIndex(list, 2), isTrue);
    expect(list.isLast(1), isFalse);
    expect(list.isLast(3), isTrue);
    expect(list.isLastIndex(1), isFalse);
    expect(list.isLastIndex(2), isTrue);
  });
```

see more in [test file](./test/is_last_test.dart).

### <a name="joinLast"></a>joinLast

The function `joinLast` it joins all elements of the list with one separator and for the last iteration a different
separator.

``` dart
  test('tests the function join.dart', () {
    final list = [1, 2, 3];
    expect(join(list, ', ', ' and '), '1, 2 and 3');
    expect(join(list), '123');
    expect(list.joinLast(', ', ' and '), '1, 2 and 3');
    expect(list.joinLast(), '123');
  });
```

see more in [test file](./test/join_test.dart).

### <a name="lastOrNull"></a>lastOrNull

The function `lastOrNull` returns last element in array, if it's empty returns null.

``` dart
  test('tests the function last_or_null.dart', () {
    final emptyList = [];
    final list = [1, 2, 3];
    expect(lastOrNull(emptyList), null);
    expect(lastOrNull(list), 3);
    expect(emptyList.lastOrNull, null);
    expect(list.lastOrNull, 3);
  });
```

see more in [test file](./test/last_or_null_test.dart).

### <a name="lastWhereOrAdd"></a>lastWhereOrAdd

The function `lastWhereOrAdd` returns the last element that satisfies the test if there isn't one add a new one and
return it.

``` dart
  test('tests the function last_where_or_add.dart', () {
    final list = [1, 2, 3];
    expect(lastWhereOrAdd(list, (value) => value == 4, () => 4), 4);
    expect(list, [1, 2, 3, 4]);
    final listE = [1, 2, 3];
    expect(listE.lastWhereOrAdd((value) => value == 4, () => 4), 4);
    expect(listE, [1, 2, 3, 4]);
  });
```

see more in [test file](./test/last_where_or_add_test.dart).

### <a name="toMap"></a>toMap

The function `toMap` returns a Map with keys that's generated by generator.

``` dart
    final list = [1, 2, 3];
    expect(toMap<String, int>(list, (value) => value.toString()),
        {'1': 1, '2': 2, '3': 3});
    expect(list.toMap((value) => "$value"), {'1': 1, '2': 2, '3': 3});
  });
```

see more in [test file](./test/to_map_test.dart).

### <a name="toStream"></a>toStream

The function `toStream` returns a stream with the items in the list.

``` dart
  test('tests the function to_stream.dart', () async {
    await expectLater([1, 2, 3].toStream(), emitsInOrder([1, 2, 3]));
    await expectLater([1, 2, 3].toStream(const Duration(milliseconds: 85)),
        emitsInOrder([1, 2, 3]));
    await expectLater(toStream([1, 2, 3]), emitsInOrder([1, 2, 3]));
  });
```

see more in [test file](./test/to_stream_test.dart).

[See another libs here](https://pub.dev/publishers/marcussoftware.info/packages)
